#ifndef POINT_HPP
#define POINT_HPP

#include "Common.hpp"

class Point
{
public:
    Point() : m_x(0), m_y(0) {}
    Point(uint32_t x, uint32_t y) : m_x(x), m_y(y) {}

    bool operator==(const Point& other) const
    {
        return m_x == other.m_x && m_y == other.m_y;
    }
    bool operator!=(const Point& other) const
    {
        return m_x != other.m_x || m_y != other.m_y;
    }

    uint32_t m_x;
    uint32_t m_y;
};

template<size_t I> const uint32_t& get(const Point& p);
template<> inline const uint32_t& get<0>(const Point& p) { return p.m_x; }
template<> inline const uint32_t& get<1>(const Point& p) { return p.m_y; }

template<size_t I> uint32_t& get(Point& p);
template<> inline uint32_t& get<0>(Point& p) { return p.m_x; }
template<> inline uint32_t& get<1>(Point& p) { return p.m_y; }

class Delta
{
public:
    Delta() : m_x(0), m_y(0) {}
    Delta(int32_t x, int32_t y) : m_x(x), m_y(y) {}

    bool operator==(const Delta& other) const
    {
        return m_x == other.m_x && m_y == other.m_y;
    }
    bool operator!=(const Delta& other) const
    {
        return m_x != other.m_x || m_y != other.m_y;
    }

    int32_t m_x;
    int32_t m_y;
};

template<size_t I> const int32_t& get(const Delta& d);
template<> inline const int32_t& get<0>(const Delta& d) { return d.m_x; }
template<> inline const int32_t& get<1>(const Delta& d) { return d.m_y; }

template<size_t I> int32_t& get(Delta& d);
template<> inline int32_t& get<0>(Delta& d) { return d.m_x; }
template<> inline int32_t& get<1>(Delta& d) { return d.m_y; }

inline Point operator+(const Point& a, const Delta& b)
{
    return Point(a.m_x + (uint32_t)b.m_x, a.m_y + (uint32_t)b.m_y);
}

inline Point operator+(const Delta& a, const Point& b)
{
    return Point((uint32_t)a.m_x + b.m_x, (uint32_t)a.m_y + b.m_y);
}

inline Delta operator+(const Delta& a, const Delta& b)
{
    return Delta(a.m_x + b.m_x, a.m_y + b.m_y);
}

inline Point operator-(const Point& a, const Delta& b)
{
    return Point(a.m_x - (uint32_t)b.m_x, a.m_y - (uint32_t)b.m_y);
}

inline Delta operator-(const Delta& a, const Delta& b)
{
    return Delta(a.m_x - b.m_x, a.m_y - b.m_y);
}

inline Delta cw(const Delta& a)
{
    return Delta(a.m_y, -a.m_x);
}

inline Delta ccw(const Delta& a)
{
    return Delta(-a.m_y, a.m_x);
}

#endif
