#include "PrintBoard.hpp"
#include "Ansi.hpp"

using std::vector;

void printBoard(Ansi& ansi,
                const Board& board,
                const vector<Point>& robotPoints,
                const Grid<uint8_t>* pWrapped,
                const vector<Point>* pReachable)
{
    uint32_t width = board.getWidth();
    uint32_t height = board.getHeight();
    for (uint32_t y = height; y > 0; )
    {
        y--;
        for (uint32_t x = 0; x < width; x++)
        {
            char code = board(x, y);
            char ch = '?';
            uint32_t fg = LightGray;
            uint32_t bg = Black;

            if (code == '#')
            {
                bg = Black;
            }
            else
            {
                bool bReachable = false;
                if (pReachable)
                {
                    for (auto& reachable : *pReachable)
                    {
                        if (Point(x, y) == reachable)
                        {
                            bReachable = true;
                            break;
                        }
                    }
                }

                if (bReachable)
                {
                    bg = Brown;
                }
                else if (pWrapped && (*pWrapped)(x, y))
                {
                    bg = Yellow;
                }
                else
                {
                    bg = LightGray;
                }
            }

            if (code == '#') { ch = ' '; fg = LightGray; }
            else if (code == ' ') { ch = code; fg = Black; }
            else if (code == 'B') { ch = code; fg = DarkGray; }
            else if (code == 'F') { ch = code; fg = Cyan; }
            else if (code == 'L') { ch = code; fg = Green; }
            else if (code == 'X') { ch = code; fg = Blue; }
            else if (code == 'R') { ch = code; fg = Magenta; }
            else if (code == 'T') { ch = code; fg = Black; }
            else if (code == 'C') { ch = code; fg = LightBlue; }
            else { ch = '?'; fg = Black; }

            bool bRobot = false;
            for (auto& robotPoint : robotPoints)
            {
                if (robotPoint == Point(x, y))
                {
                    bRobot = true;
                    break;
                }
            }
            if (bRobot) { ch = '@'; fg = Red; }

            ansi.changeColor(fg, bg, false /*ch == ' '*/);
            printf("%c", ch);
        }
        ansi.changeColor(LightGray, Black);
        printf("\n");
    }
}
