#ifndef PROCESS_HPP
#define PROCESS_HPP

#include "Common.hpp"

class ProcessImpl;

class Process
{
public:
    Process();
    Process(Process&& other);
    ~Process();
    Process& operator=(Process&& other);
    void swap(Process& other);

    bool create(const std::string& program,
                const std::vector<std::string>& args,
                bool swallowStdErr,
                std::string* pMsg = nullptr);

    bool write(const void* buf,
               size_t size,
               size_t* pBytesWritten,
               std::string* pMsg = nullptr);
    bool writeUntilMS(const void* buf,
                      size_t size,
                      size_t* pBytesWritten,
                      uint64_t timeMS,
                      bool* pTimeout,
                      std::string* pMsg = nullptr);
    bool writeForMS(const void* buf,
                    size_t size,
                    size_t* pBytesWritten,
                    uint64_t durationMS,
                    bool* pTimeout,
                    std::string* pMsg = nullptr);
    void writeClose();

    bool read(void* buf,
              size_t size,
              size_t* pBytesRead,
              std::string* pMsg = nullptr);
    bool readUntilMS(void* buf,
                     size_t size,
                     size_t* pBytesRead,
                     uint64_t timeMS,
                     bool* pTimeout,
                     std::string* pMsg = nullptr);
    bool readForMS(void* buf,
                   size_t size,
                   size_t* pBytesRead,
                   uint64_t durationMS,
                   bool* pTimeout,
                   std::string* pMsg = nullptr);
    void readClose();

    void wait();
    void waitUntilMS(uint64_t timeMS,
                     bool* pTimeout);
    void waitForMS(uint64_t durationMS,
                   bool* pTimeout);

    void kill();

private:
    std::unique_ptr<ProcessImpl> m_pImpl;

private:
    Process(const Process& other) = delete;
    Process& operator=(const Process& other) = delete;
};

void swap(Process& x, Process& y);

#endif
