#include "Puzzle.hpp"

using std::string;
using std::vector;


namespace
{
    bool parseUInt(const string& data,
                   size_t& pos,
                   uint32_t *pVal,
                   string* pMsg = nullptr)
    {
        size_t origPos = pos;
        size_t size = data.size();
        uint32_t val = 0;
        while (true)
        {
            if (pos >= size) break;
            char ch = data[pos];
            if (ch < '0' || ch > '9') break;
            if (val >= 100000000)
            {
                if (pMsg) *pMsg = "Number too large";
                return false;
            }
            val *= 10;
            val += (ch - '0');
            pos++;
        }
        if (pos == origPos)
        {
            if (pMsg) *pMsg = "No digits in number";
            return false;
        }
        *pVal = val;
        return true;
    }

    bool expectChar(const string& data,
                    size_t& pos,
                    char expectedCh,
                    string* pMsg = nullptr)
    {
        size_t size = data.size();
        if (pos >= size)
        {
            if (pMsg) *pMsg = string("Expected '") + expectedCh + "' at end of input";
            return false;
        }

        if (data[pos] != expectedCh)
        {
            if (pMsg) *pMsg = string("Expected '") + expectedCh + "'";
            return false;
        }

        pos++;
        return true;
    }

    bool parsePoint(const string& data,
                    size_t& pos,
                    Point* pPoint,
                    string* pMsg)
    {
        if (!expectChar(data, pos, '(', pMsg)) return false;

        uint32_t x = 0;
        if (!parseUInt(data, pos, &x, pMsg)) return false;

        if (!expectChar(data, pos, ',', pMsg)) return false;

        uint32_t y = 0;
        if (!parseUInt(data, pos, &y, pMsg)) return false;

        if (!expectChar(data, pos, ')', pMsg)) return false;

        *pPoint = Point(x, y);
        return true;
    }

    bool parsePoints(const string& data,
                     size_t& pos,
                     vector<Point>* pPoints,
                     string* pMsg)
    {
        pPoints->clear();
        size_t size = data.size();
        while (pos < size && data[pos] == '(')
        {
            Point point;
            if (!parsePoint(data, pos, &point, pMsg)) return false;
            pPoints->push_back(std::move(point));

            if (pos >= size || data[pos] != ',')
            {
                break;
            }
            pos++;
        }
        return true;
    }

}

bool parsePuzzle(const string& data,
                 Puzzle* pPuzzle,
                 string* pMsg)
{
    size_t pos = 0;

    if (!parseUInt(data, pos, &pPuzzle->m_bNum, pMsg)) return false;
    if (!expectChar(data, pos, ',', pMsg)) return false;
    if (!parseUInt(data, pos, &pPuzzle->m_eNum, pMsg)) return false;
    if (!expectChar(data, pos, ',', pMsg)) return false;
    if (!parseUInt(data, pos, &pPuzzle->m_tSize, pMsg)) return false;
    if (!expectChar(data, pos, ',', pMsg)) return false;
    if (!parseUInt(data, pos, &pPuzzle->m_vMin, pMsg)) return false;
    if (!expectChar(data, pos, ',', pMsg)) return false;
    if (!parseUInt(data, pos, &pPuzzle->m_vMax, pMsg)) return false;
    if (!expectChar(data, pos, ',', pMsg)) return false;
    if (!parseUInt(data, pos, &pPuzzle->m_mNum, pMsg)) return false;
    if (!expectChar(data, pos, ',', pMsg)) return false;
    if (!parseUInt(data, pos, &pPuzzle->m_fNum, pMsg)) return false;
    if (!expectChar(data, pos, ',', pMsg)) return false;
    if (!parseUInt(data, pos, &pPuzzle->m_dNum, pMsg)) return false;
    if (!expectChar(data, pos, ',', pMsg)) return false;
    if (!parseUInt(data, pos, &pPuzzle->m_rNum, pMsg)) return false;
    if (!expectChar(data, pos, ',', pMsg)) return false;
    if (!parseUInt(data, pos, &pPuzzle->m_cNum, pMsg)) return false;
    if (!expectChar(data, pos, ',', pMsg)) return false;
    if (!parseUInt(data, pos, &pPuzzle->m_xNum, pMsg)) return false;
    if (!expectChar(data, pos, '#', pMsg)) return false;
    if (!parsePoints(data, pos, &pPuzzle->m_iSqs, pMsg)) return false;
    if (!expectChar(data, pos, '#', pMsg)) return false;
    if (!parsePoints(data, pos, &pPuzzle->m_oSqs, pMsg)) return false;

    return true;
}
