#ifndef SIM_HPP
#define SIM_HPP

#include "Common.hpp"
#include "Problem.hpp"
#include "Solution.hpp"

class SimRobot
{
public:
    Point m_point;
    std::vector<Delta> m_deltas;
    std::vector<Point> m_reachable;
    uint32_t m_timeF = 0;
    uint32_t m_timeL = 0;
    uint32_t m_routeIdx = 0;
};

class Sim
{
public:
    Sim();
    Sim(Board board,
        const Point& startPoint,
        const std::string& buy,
        Solution solution);
    Sim(const Sim& other) = delete;
    Sim(Sim&& other) = delete;
    ~Sim();
    Sim& operator=(const Sim& other) = delete;
    Sim& operator=(Sim&& other) = delete;

    void init(Board board,
              const Point& startPoint,
              const std::string& buy,
              Solution solution);

    void step();
    void run();

private:
    void pickUp(SimRobot& robot);
    void wrap(SimRobot& robot);
    void checkDone();

public:
    Board m_board;
    Solution m_solution;
    std::vector<SimRobot> m_robots;
    Grid<uint8_t> m_wrapped;
    uint32_t m_time;
    uint32_t m_unwrapped;
    uint32_t m_countB;
    uint32_t m_countF;
    uint32_t m_countL;
    uint32_t m_countR;
    uint32_t m_countC;
    bool m_invalid;
    bool m_done;
};

#endif
