#ifndef SOLUTION_HPP
#define SOLUTION_HPP

#include "Common.hpp"
#include "Point.hpp"

class Action
{
public:
    Action() : m_code(0), m_x(0), m_y(0) {}
    explicit Action(char code) : m_code(code), m_x(0), m_y(0) {}
    Action(char code, const Point& point) :
        m_code(code),
        m_x(point.m_x),
        m_y(point.m_y)
    {}
    Action(char code, const Delta& delta) :
        m_code(code),
        m_x((uint32_t)delta.m_x),
        m_y((uint32_t)delta.m_y)
    {}

    Point getPoint() const { return Point(m_x, m_y); }
    Delta getDelta() const { return Delta((int32_t)m_x, (int32_t)m_y); }

    bool operator==(const Action& other) const
    {
        if (m_code != other.m_code) return false;
        if (m_code == 'B' && getDelta() != other.getDelta()) return false;
        if (m_code == 'T' && getPoint() != other.getPoint()) return false;
        return true;
    }
    bool operator!=(const Action& other) const { return !(*this == other); }

    char m_code;
    uint32_t m_x;
    uint32_t m_y;
    uint32_t m_padding;
};

typedef std::vector<Action> Route;
typedef std::vector<Route> Solution;

std::string formatAction(const Action& action);
std::string formatRoute(const Route& route);
std::string formatSolution(const Solution& solution);

bool parseSolution(const std::string& data,
                   Solution* pSolution,
                   std::string* pMsg = nullptr);

#endif
