#ifndef SOLVER_HPP
#define SOLVER_HPP

#include "Common.hpp"
#include "Grid.hpp"
#include "Problem.hpp"
#include "Solution.hpp"

class SolverCell
{
public:
    uint32_t m_visited = 0;
    Point m_point;
    char m_code = 0;
};

class MissionCell
{
public:
    uint32_t m_visited = 0;
    uint32_t m_dist = 0;
};

class SolverRobot
{
public:
    Grid<SolverCell> m_grid;
    std::vector<Point> m_oldPoints;
    std::vector<Point> m_newPoints;
    std::vector<Delta> m_deltas;
    std::vector<Delta> m_cwDeltas;
    std::vector<Delta> m_ccwDeltas;
    Grid<MissionCell> m_missionGrid;
    std::vector<Point> m_missionOldPoints;
    std::vector<Point> m_missionNewPoints;
    uint32_t m_timeF = 0;
    uint32_t m_timeL = 0;
    uint32_t m_visited = 0;
    uint32_t m_missionVisited = 0;
    char m_code = 0;
    bool m_expiredF = false;
    bool m_onMission = false;
};

class PlanCell
{
public:
    uint8_t m_noise = 0;
    int8_t m_bias = 0;
};

class Plan
{
public:
    Grid<PlanCell> m_grid;
};

class Solver
{
public:
    Solver();
    Solver(Board board,
           const Point& startPoint,
           const std::string& buy,
           const Plan* pPlan = nullptr);
    Solver(const Solver& other) = delete;
    Solver(Solver&& other) = delete;
    ~Solver();
    Solver& operator=(const Solver& other) = delete;
    Solver& operator=(Solver&& other) = delete;

    void init(Board board,
              const Point& startPoint,
              const std::string& buy,
              const Plan* pPlan = nullptr);

    void step();
    void run();

private:
    void pickUpStart(SolverRobot& robot, Point point);
    void pickUpFinish(SolverRobot& robot);
    void wrap(SolverRobot& robot,
              const std::vector<Delta>& deltas,
              Point point,
              bool test = false,
              bool test2 = false);
    void updateMissions(SolverRobot& robot);

public:
    Board m_board;
    Solution m_solution;
    std::vector<SolverRobot> m_robots;
    Grid<uint8_t> m_wrapped;
    std::vector<Point> m_wrapPoints;
    std::vector<Point> m_pointsC;
    std::vector<Point> m_pointsX;
    std::vector<Point> m_pointsB;
    const Plan* m_pPlan;
    uint32_t m_time;
    uint32_t m_unwrapped;
    uint32_t m_countB;
    uint32_t m_countF;
    uint32_t m_countL;
    uint32_t m_countR;
    uint32_t m_countC;
    bool m_needMissionUpdate;
};

#endif
