#include "Common.hpp"
#include "StringUtils.hpp"
#include "FileUtils.hpp"
#include "Problem.hpp"
#include "Solution.hpp"
#include "Sim.hpp"
#include <cmath>

using std::string;
using std::vector;

class ProblemData
{
};

int main(int argc, char *argv[])
{
    string msg;

    vector<string> buys{"", "C"};
    uint32_t numBuys = buys.size();
    uint32_t coins = 400142;

    vector<uint32_t> costs(numBuys);
    for (uint32_t iBuy = 0; iBuy < numBuys; iBuy++)
    {
        auto& buy = buys[iBuy];
        uint32_t cost = 0;
        for (char code : buy)
        {
            if (code == 'B') cost += 1000;
            else if (code == 'F') cost += 300;
            else if (code == 'L') cost += 700;
            else if (code == 'R') cost += 1200;
            else if (code == 'C') cost += 2000;
        }
        costs[iBuy] = cost;
    }

    vector<vector<uint32_t>> times(301);
    vector<uint32_t> scores(301);
    vector<uint32_t> bestGrossGains(301);
    vector<uint32_t> bestNetGains(301);
    vector<uint32_t> bestCosts(301);
    vector<uint32_t> bestBuys(301);
    vector<double> bestGainPerCoins(301);
    vector<uint32_t> chosenBuys(301);

    for (uint32_t iProblem = 1; iProblem <= 300; iProblem++)
    {
        string problemFile = strprintf("../problems/prob-%03" PRIu32 ".desc", iProblem);

        string problemData;
        if (!readFile(problemFile, &problemData))
        {
            printf("%03" PRIu32 ": Error reading problem file\n", iProblem);
            return 1;
        }

        Board board;
        Point startPoint;
        if (!parseProblem(problemData, &board, &startPoint, &msg))
        {
            printf("%03" PRIu32 ": %s\n", iProblem, msg.c_str());
            return 1;
        }

        {
            uint32_t width = board.getWidth();
            uint32_t height = board.getHeight();
            double safetyMargin = 0.7;
            uint32_t score = (uint32_t)std::ceil(1000.0 * std::log2((double)(width * height)) * safetyMargin);
            scores[iProblem] = score;
            //printf("%03" PRIu32 ": %" PRIu32 "\n", iProblem, score);
            //continue;
        }

        times[iProblem].resize(numBuys);

        for (uint32_t iBuy = 0; iBuy < numBuys; iBuy++)
        {
            auto& buy = buys[iBuy];
            string solutionFile = strprintf("../solutions%s/prob-%03" PRIu32 ".sol", buy.c_str(), iProblem);

            string solutionData;
            if (!readFile(solutionFile, &solutionData))
            {
                if (iBuy == 0)
                {
                    printf("%03" PRIu32 ": '%s': Error reading solution file\n", iProblem, buy.c_str());
                    return 1;
                }
                continue;
            }

            Solution solution;
            if (!parseSolution(solutionData, &solution, &msg))
            {
                printf("%03" PRIu32 ": %s\n", iProblem, msg.c_str());
                return 1;
            }

            Sim sim(board, startPoint, buy, solution);
            sim.run();

            if (sim.m_invalid)
            {
                printf("%03" PRIu32 ": '%s': Best solution invalid\n", iProblem, buy.c_str());
                return 1;
            }

            if (sim.m_unwrapped != 0)
            {
                printf("%03" PRIu32 ": '%s': Best solution incomplete\n", iProblem, buy.c_str());
                return 1;
            }

            times[iProblem][iBuy] = sim.m_time;

            //printf("%03" PRIu32 ": '%s': %" PRIu32 "\n", iProblem, buy.c_str(), sim.m_time);
        }

        for (uint32_t iBuy = 1; iBuy < numBuys; iBuy++)
        {
            uint32_t cost = costs[iBuy];
            if (times[iProblem][iBuy] >= times[iProblem][0]) continue;
            uint32_t grossGain = scores[iProblem] * (times[iProblem][0] - times[iProblem][iBuy]) / times[iProblem][0];
            uint32_t netGain = grossGain <= cost ? 0 : grossGain - cost;
            //printf("%03" PRIu32 ": '%s': %" PRIu32 " %" PRIu32 "\n", iProblem, buys[iBuy].c_str(), grossGain, netGain);
            if (netGain == 0) continue;
            double gainPerCoin = (double)netGain / (double)cost;
            if (bestBuys[iProblem] == 0 ||
                gainPerCoin > bestGainPerCoins[iProblem])
            {
                bestGrossGains[iProblem] = grossGain;
                bestNetGains[iProblem] = netGain;
                bestCosts[iProblem] = cost;
                bestBuys[iProblem] = iBuy;
                bestGainPerCoins[iProblem] = gainPerCoin;
            }
        }
    }

    vector<uint32_t> idxs(300);
    for (uint32_t iProblem = 1; iProblem <= 300; iProblem++)
    {
        idxs[iProblem - 1] = iProblem;
    }

    std::sort(idxs.begin(), idxs.end(),
              [&](uint32_t iProblemA, uint32_t iProblemB) -> bool
              {
                  double gainPerCoinA = bestGainPerCoins[iProblemA];
                  double gainPerCoinB = bestGainPerCoins[iProblemB];
                  if (gainPerCoinA != gainPerCoinB)
                  {
                      return gainPerCoinA > gainPerCoinB;
                  }
                  return iProblemA < iProblemB;
              }
        );

    uint32_t coinsSpent = 0;
    uint32_t coinsLeft = coins;
    uint32_t totalNetGain = 0;
    for (uint32_t idx = 0; idx < 300; idx++)
    {
        uint32_t iProblem = idxs[idx];
        uint32_t iBuy = bestBuys[iProblem];
        uint32_t netGain = bestNetGains[iProblem];
        //printf("%03" PRIu32 ": '%s': %" PRIu32 "\n", iProblem, buys[iBuy].c_str(), netGain);
        uint32_t cost = costs[iBuy];
        if (netGain > 0 && cost <= coinsLeft)
        {
            printf("%03" PRIu32 ": '%s': %" PRIu32 "\n", iProblem, buys[iBuy].c_str(), netGain);
            coinsSpent += cost;
            coinsLeft -= cost;
            totalNetGain += netGain;
            chosenBuys[iProblem] = iBuy;
        }
        else
        {
            chosenBuys[iProblem] = 0;
        }
    }
    printf("Coins spent: %" PRIu32 "\n", coinsSpent);
    printf("Coins left: %" PRIu32 "\n", coinsLeft);
    printf("Total net gain: %" PRIu32 "\n", totalNetGain);

#if 1
    for (uint32_t iProblem = 1; iProblem <= 300; iProblem++)
    {
        uint32_t iBuy = chosenBuys[iProblem];
        auto& buy = buys[iBuy];
        string solutionFile = strprintf("../solutions%s/prob-%03" PRIu32 ".sol", buy.c_str(), iProblem);
        string assembleFile = strprintf("../assemble/prob-%03" PRIu32 ".sol", iProblem);
        string buyFile = strprintf("../assemble/prob-%03" PRIu32 ".buy", iProblem);

        string solutionData;
        if (!readFile(solutionFile, &solutionData))
        {
            printf("%03" PRIu32 ": '%s': Error reading solution file\n", iProblem, buy.c_str());
            return 1;
        }

        if (!writeFile(assembleFile, solutionData))
        {
            printf("%03" PRIu32 ": Error writing solution file\n", iProblem);
            return 1;
        }

        if (iBuy != 0)
        {
            string buyData = buy + '\n';
            if (!writeFile(buyFile, buyData))
            {
                printf("%03" PRIu32 ": Error writing buy file\n", iProblem);
                return 1;
            }
        }
    }
#endif

    return 0;
}
