#include "Common.hpp"
#include "StringUtils.hpp"
#include "FileUtils.hpp"
#include "Problem.hpp"
#include "Solution.hpp"
#include "Sim.hpp"
#include "Ansi.hpp"
#include "PrintBoard.hpp"
#include "PrintSim.hpp"

using std::string;
using std::vector;

int main(int argc, char *argv[])
{
    string msg;

#if 0
    Ansi ansi;
    ansi.changeColor(LightGray, Black);
    for (uint32_t i = 1; i <= 300; i++)
    {
        string problemFile = strprintf("../problems/prob-%03" PRIu32 ".desc", i);
        //string problemFile = "../../../part-1-examples/example-01.desc";

        string data;
        if (!readFile(problemFile, &data))
        {
            printf("Error reading %s\n", problemFile.c_str());
            return 1;
        }

        Board board;
        Point startPoint;
        if (!parseProblem(data, &board, &startPoint, &msg))
        {
            printf("%03" PRIu32 ": %s\n", i, msg.c_str());

            return 1;
        }

#if 0
        vector<Point> robotPoints(1, startPoint);
        printf("%03" PRIu32 ":\n", i);
        printBoard(ansi, board, robotPoints);
        printf("\n");
#endif

#if 1
        vector<uint32_t> counts(256);
        uint32_t width = board.getWidth();
        uint32_t height = board.getHeight();
        for (uint32_t y = 0; y < height; y++)
        {
            for (uint32_t x = 0; x < width; x++)
            {
                counts[(uint8_t)board(x, y)]++;
            }
        }
        printf("%03" PRIu32 ": %3" PRIu32 " %3" PRIu32 " %3" PRIu32 " %3" PRIu32 " %3" PRIu32 " %3" PRIu32 "\n",
               i,
               counts['B'],
               counts['F'],
               counts['L'],
               counts['X'],
               counts['R'],
               counts['C']);
#endif
    }

    printf("Done\n");
#endif

#if 1
    {
        //string buyFile = "";
        //string problemFile = "../examples/example-01.desc";
        //string solutionFile = "../examples/example-01-1.sol";
        //string solutionFile = "../examples/example-01-2.sol";
        //string solutionFile = "../examples/example-01-3.sol";
        //string problemFile = "../examples/example-02.desc";
        //string solutionFile = "../examples/example-02-1.sol";
        //string problemFile = "../examples/example-03.desc";
        //string solutionFile = "../examples/example-03-1.sol";
        //string problemFile = "../examples/example-04.desc";
        //string solutionFile = "../examples/example-04-1.sol";
        //string solutionFile = "../examples/example-04-2.sol";
        //string buyFile = "../examples/example-04.buy";
        ///string problemFile = "../tests/wrap-after-drill.desc";
        //string solutionFile = "../tests/wrap-after-drill.sol";
        //string problemFile = "../tests/wrap-after-manipulator.desc";
        //string solutionFile = "../tests/wrap-after-manipulator.sol";
        //string problemFile = "../tests/fast-wheels-time.desc";
        //string solutionFile = "../tests/fast-wheels-time.sol";
        //string problemFile = "../tests/wrap-after-clone.desc";
        //string solutionFile = "../tests/wrap-after-clone-1.sol";
        //string solutionFile = "../tests/wrap-after-clone-2.sol";
        //string solutionFile = "../tests/wrap-after-clone-3.sol";
        //string problemFile = "../tests/wrap-before-start.desc";
        //string solutionFile = "../tests/wrap-before-start-1.sol";
        //string solutionFile = "../tests/wrap-before-start-2.sol";
        //string problemFile = "../tests/fast-wheels-pick-up.desc";
        //string solutionFile = "../tests/fast-wheels-pick-up.sol";
        string problemFile = "../problems/prob-209.desc";
        string solutionFile = "../bad/prob-209.sol";
        string buyFile = "../bad/prob-209.buy";

        string problemData;
        if (!readFile(problemFile, &problemData))
        {
            printf("Error reading problem file\n");
            return 1;
        }

        string solutionData;
        if (!readFile(solutionFile, &solutionData))
        {
            printf("Error reading solution file\n");
            return 1;
        }

        string buy;
        if (!buyFile.empty())
        {
            if (!readFile(buyFile, &buy))
            {
                printf("Error reading buy file\n");
                return 1;
            }
        }

        Board board;
        Point startPoint;
        if (!parseProblem(problemData, &board, &startPoint, &msg))
        {
            printf("%s\n", msg.c_str());
            return 1;
        }

        Solution solution;
        if (!parseSolution(solutionData, &solution, &msg))
        {
            printf("%s\n", msg.c_str());
            return 1;
        }

        Sim sim(board, startPoint, buy, solution);

        Ansi ansi;
        ansi.changeColor(LightGray, Black);

        printSim(ansi, sim);
        printf("Time: %" PRIu32 "\n", sim.m_time);
        printf("\n");

        while (!sim.m_done)
        {
            //printf("Action: %s\n", formatAction(action).c_str());

            sim.step();

            if (sim.m_invalid)
            {
                printf("Solution invalid\n");
                return 1;
            }

            printSim(ansi, sim);
            printf("Time: %" PRIu32 "\n", sim.m_time);
            printf("\n");
        }

        if (sim.m_unwrapped != 0)
        {
            printf("Solution incomplete\n");
        }
        else
        {
            printf("Solution ok\n");
        }
    }
#endif

    return 0;
}
